#ifndef QWKCONFIG_H
#define QWKCONFIG_H

// Caution: This file is generated by CMake automatically during configure.
// WARNING!!! DO NOT EDIT THIS FILE MANUALLY!!!
// ALL YOUR MODIFICATIONS HERE WILL GET LOST AFTER RE-CONFIGURING!!!

// SHA256: 5eedba1d60d689379aac55a9b6e19aefaf5d77f89fe4e633de5642b1f2d87e86

#define QWINDOWKIT_ENABLE_QT_WINDOW_CONTEXT -1
#define QWINDOWKIT_ENABLE_STYLE_AGENT 1
#define QWINDOWKIT_ENABLE_WINDOWS_SYSTEM_BORDERS 1

#endif // QWKCONFIG_H
